/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmRecorder.cpp
 * version 5
 */
#include <sstream>
#include "TsmRecorder.h"
#include "Tsm.h"
#include "TsmClass.h"
#include "TsmPort.h"
#include "TsmSupervisor.h"
using namespace std;

namespace TsmRTE
{
  int TsmRecorder::NumberOfUnhandledFunctions = 0;
  int TsmRecorder::NumberOfUnvisitedStates = 0;
  int TsmRecorder::NumberOfUnvisitedTransitions = 0;
  int TsmRecorder::NumberOfSyncReturnErrors = 0;
  int TsmRecorder::NumberOfQueueOverflows = 0;
  
  vector<string>  TsmRecorder::Records;
  int             TsmRecorder::NrCalls = 0;
  bool            TsmRecorder::bEnable = false;
  const string    TsmRecorder::Separator = "============================================================";

  void TsmRecorder::Enable()
  {
    bEnable = true;
  }

  void TsmRecorder::AddNative( string sNative )
  {
    string s = "NaTr: " + sNative;
    Records.push_back( s );
  }

  void TsmRecorder::AddCall( string sCall, TsmClass* oSource, TsmPort* oPort )
  {
    TsmClass* lTarget = oPort->Target;
    string s = "Call: " + sCall + " @ " + oSource->FullName() + " -> " + lTarget->FullName();
    Records.push_back( s );
    if ( oSource->IsMock || lTarget->IsMock )
    {
      NrCalls++;
    }
    stringstream ss;
    ss << "C," << sCall << "," << oSource->SeqId << "," << lTarget->SeqId << "," << oPort->ToSeq();
    Tsm::Seq( ss.str() );
  }

  void TsmRecorder::AddStates( string sStates )
  {
    string s = "States: " + sStates;
    Records.push_back( s );
  }

  void TsmRecorder::UnhandledFunction( string sEvent )
  {
    NumberOfUnhandledFunctions++;
    Show( "**** UNHANDLED FUNCTION : " + sEvent );
  }

  void TsmRecorder::UnvisitedState( string sEvent )
  {
    stringstream sb;

    NumberOfUnvisitedStates++;
    sb << Separator << endl;
    sb << "Unvisited state: " << sEvent << endl;
    Tsm::Log( sb.str() );
  }

  void TsmRecorder::UnvisitedTransition( string sState, string sEvent )
  {
    stringstream sb;

    NumberOfUnvisitedTransitions++;
    sb << Separator << endl;
    sb << "@ state: " + sState << endl;
    sb << "Unvisited transition: " + sEvent << endl;
    Tsm::Log( sb.str() );
  }

  void TsmRecorder::SyncReturnError( string sEvent )
  {
    NumberOfSyncReturnErrors++;
    Show( sEvent );
  }

  void TsmRecorder::QueueOverflow()
  {
    NumberOfQueueOverflows++;
    Show( "**** QUEUE OVERFLOW " );
  }

  void TsmRecorder::Show( string sEvent )
  {
    if ( bEnable )
    {
      stringstream sb;
      sb << Separator << endl;
      sb << sEvent << endl;

      for ( unsigned int i=0; i<Records.size(); i++ )
      {
        sb << Records[i] << endl;
      }

      sb << TsmSupervisor::SnappedClasses();
      Tsm::Log( sb.str() );
    }
    Tsm::Seq( "X," + sEvent );
  }

  void TsmRecorder::Show( string sEvent, int iNrRecords )
  {
    if ( bEnable )
    {
      stringstream sb;
      sb << Separator << endl;
      sb << sEvent << endl;

      for ( int i=0; i<iNrRecords; i++ )
      {
        sb << Records[i] << endl;
      }
      sb << "=== start of cycle ===" << endl;
      for ( unsigned int i=iNrRecords; i<Records.size(); i++ )
      {
        sb << Records[i] << endl;
      }

      sb << TsmSupervisor::GetCurrentSituation();

      Tsm::Log( sb.str() );
    }
    Tsm::Seq( "X," + sEvent );
  }
}
