/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmSupervisor.h
 * version 3
 */
#ifndef TSM_SUPERVISOR_H
#define TSM_SUPERVISOR_H

#include <boost/thread/thread.hpp>
#include "TsmActivity.h"
#include "TsmThreadState.h"
#include "TsmThreadType.h"
using namespace std;

namespace TsmRTE
{
  class TsmCbThread;
  class TsmIntQueue;
  class TsmThread;
  class TsmClass;

  class TsmSupervisor
  {
  private:
    static boost::mutex  LockThreads;
    static TsmActivity*  mActivity;
    static TsmCbThread*  mCbThread;
    static bool          mbVerify;

  private:
    static int      mMaximumQueueSize;

  public:
    static int      mRandomSeed;
    static long     mMaxNrSimLoops;

  public:
    static void Initialise();
    static void Terminate();
    static TsmIntQueue* GetIntQueue();
    static void Announce();
    static void Abandon();
    static vector<TsmClass*> * GetClasses();
    static int Register( TsmThread* oTsmThread );
    static int Register( TsmThread* oTsmThread, TsmThreadType eType );
    static void SetState( boost::thread::id ThreadBid );
    static void SetState( TsmThreadState eState );
    static void SetState( boost::thread::id ThreadBid, TsmThreadState eState );
    static void NativeEntry( TsmClass* oOwner, const int aNatives[], int NrNatives );
    static void NativeExit( TsmClass* oOwner, const int aNatives[], int NrNatives );
    static string SnappedClasses();
    static string SnappedThreads();
    static string GetCurrentSituation();

    static bool QueueOverflow( int QueueCount );
    static bool ChoiceOK( int MaxNrChoices, int ChoiceIndex, string sGuard );
    static void Verify();
    static void ConfigureSimulation( int iSeed, long iNrLoops );
    static void Simulate();
    static void Summary();
    static void Cleanup();
  };
}
#endif
