#---------------------------------------------------------
# File: app-rules.mak
#
# Purpose: Application make rules. This file contains common make rules to
#          be included by application make files. 
#
#---------------------------------------------------------

# OS_LINUX:
BOOST_LIBS_DIR = /usr/lib
BOOST_THREAD_LIB = -lboost_thread
# OS_WINDOWS:
#BOOST_LIBS_DIR = "C:\Program Files\boost\boost_1_47_0\stage\lib"
#BOOST_THREAD_LIB = -lboost_thread-mgw45-mt-1_47

#---------------------------------------------------------
# Default rule for building the App Loadable file
#---------------------------------------------------------
default: $(APPTARGET)

#---------------------------------------------------------
# Default Compiler rule
#---------------------------------------------------------
.cpp.o:
	$(COMPILERCPP) $(INCLUDE_PATH) $(COPTS) $(DEBUG_OPTS) -DOS_DEBUG_LEVEL=$(DEBUG_LEVEL) -c -o $@ $<

.c.o:
	$(COMPILER) $(INCLUDE_PATH) $(COPTS) $(DEBUG_OPTS) -DOS_DEBUG_LEVEL=$(DEBUG_LEVEL) -c -o $@ $<

#---------------------------------------------------------
# Default Assembly Code Rule
#---------------------------------------------------------
.s.o:
	$(COMPILER) $(INCLUDE_PATH) $(ASOPTS) $(COPTS) $(DEBUG_OPTS) -DOS_DEBUG_LEVEL=$(DEBUG_LEVEL) -c -o $@ $<

#---------------------------------------------------------
# Rule to generate the dependency list
#---------------------------------------------------------
depend: $(SOURCES)
	$(COMPILER) -MM $(INCLUDE_PATH) $(DEBUG_OPTS) -DOS_DEBUG_LEVEL=$(DEBUG_LEVEL) -c $^ > $(APPTARGET).d

#---------------------------------------------------------
# Include the rule for linking the application.
#---------------------------------------------------------

#---------------------------------------------------------
# Linker flags that are needed
#---------------------------------------------------------
LDFLAGS = -L$(ARCHIVE_DIR) -L$(BOOST_LIBS_DIR)

#---------------------------------------------------------
# Core Link Rule
#---------------------------------------------------------
$(APPTARGET): $(OBJS) $(OBJSCPP)
	$(COMPILERCPP) $(DEBUG_FLAGS) -o $(APPTARGET) $(OBJS) $(OBJSCPP) $(LDFLAGS) $(LIBS) $(BOOST_THREAD_LIB)
	
#---------------------------------------------------------
# Default rule for installing app
#---------------------------------------------------------
install:
	$(CP) $(APPTARGET) $(EXE_DIR)

#---------------------------------------------------------
# Rule for cleaning
# All of the possible extensions are included here so we can clean up after
# an old build from a different architecture.
#---------------------------------------------------------
clean:
	echo cleaning -$(RM)
	-$(RM) *.d
	-$(RM) *.o
	-$(RM) *.a
	-$(RM) $(APPTARGET)
	-$(RM) $(APPTARGET).exe

#---------------------------------------------------------
# Include the dependency list
#---------------------------------------------------------
-include $(APPTARGET).d
